@@ -4,18 +4,26 @@
  *
  * This file is part of libtiled.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ *    1. Redistributions of source code must retain the above copyright notice,
+ *       this list of conditions and the following disclaimer.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program. If not, see <http://www.gnu.org/licenses/>.
+ *    2. Redistributions in binary form must reproduce the above copyright
+ *       notice, this list of conditions and the following disclaimer in the
+ *       documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
+ * EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+ * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+ * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+ * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "gidmapper.h"
@@ -37,14 +45,14 @@ GidMapper::GidMapper()
 
 GidMapper::GidMapper(const QList<Tileset *> &tilesets)
 {
-    uint firstGid = 1;
+    unsigned firstGid = 1;
     foreach (Tileset *tileset, tilesets) {
         insert(firstGid, tileset);
         firstGid += tileset->tileCount();
     }
 }
 
-Cell GidMapper::gidToCell(uint gid, bool &ok) const
+Cell GidMapper::gidToCell(unsigned gid, bool &ok) const
 {
     Cell result;
 
@@ -64,7 +72,7 @@ Cell GidMapper::gidToCell(uint gid, bool &ok) const
         ok = false;
     } else {
         // Find the tileset containing this tile
-        QMap<uint, Tileset*>::const_iterator i = mFirstGidToTileset.upperBound(gid);
+        QMap<unsigned, Tileset*>::const_iterator i = mFirstGidToTileset.upperBound(gid);
         --i; // Navigate one tileset back since upper bound finds the next
         int tileId = gid - i.key();
         const Tileset *tileset = i.value();
@@ -89,7 +97,7 @@ Cell GidMapper::gidToCell(uint gid, bool &ok) const
     return result;
 }
 
-uint GidMapper::cellToGid(const Cell &cell) const
+unsigned GidMapper::cellToGid(const Cell &cell) const
 {
     if (cell.isEmpty())
         return 0;
@@ -97,15 +105,15 @@ uint GidMapper::cellToGid(const Cell &cell) const
     const Tileset *tileset = cell.tile->tileset();
 
     // Find the first GID for the tileset
-    QMap<uint, Tileset*>::const_iterator i = mFirstGidToTileset.begin();
-    QMap<uint, Tileset*>::const_iterator i_end = mFirstGidToTileset.end();
+    QMap<unsigned, Tileset*>::const_iterator i = mFirstGidToTileset.begin();
+    QMap<unsigned, Tileset*>::const_iterator i_end = mFirstGidToTileset.end();
     while (i != i_end && i.value() != tileset)
         ++i;
 
     if (i == i_end) // tileset not found
         return 0;
 
-    uint gid = i.key() + cell.tile->id();
+    unsigned gid = i.key() + cell.tile->id();
     if (cell.flippedHorizontally)
         gid |= FlippedHorizontallyFlag;
     if (cell.flippedVertically)
