@@ -4,18 +4,26 @@
  *
  * This file is part of libtiled.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ *    1. Redistributions of source code must retain the above copyright notice,
+ *       this list of conditions and the following disclaimer.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program. If not, see <http://www.gnu.org/licenses/>.
+ *    2. Redistributions in binary form must reproduce the above copyright
+ *       notice, this list of conditions and the following disclaimer in the
+ *       documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
+ * EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+ * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+ * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+ * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #ifndef TILED_GIDMAPPER_H
@@ -47,7 +55,7 @@ public:
     /**
      * Insert the given \a tileset with \a firstGid as its first global ID.
      */
-    void insert(uint firstGid, Tileset *tileset)
+    void insert(unsigned firstGid, Tileset *tileset)
     { mFirstGidToTileset.insert(firstGid, tileset); }
 
     /**
@@ -64,13 +72,13 @@ public:
      * Returns the cell data matched by the given \a gid. The \a ok parameter
      * indicates whether an error occurred.
      */
-    Cell gidToCell(uint gid, bool &ok) const;
+    Cell gidToCell(unsigned gid, bool &ok) const;
 
     /**
      * Returns the global tile ID for the given \a cell. Returns 0 when the
      * cell is empty or when its tileset isn't known.
      */
-    uint cellToGid(const Cell &cell) const;
+    unsigned cellToGid(const Cell &cell) const;
 
     /**
      * This sets the original tileset width. In case the image size has
@@ -80,7 +88,7 @@ public:
     void setTilesetWidth(const Tileset *tileset, int width);
 
 private:
-    QMap<uint, Tileset*> mFirstGidToTileset;
+    QMap<unsigned, Tileset*> mFirstGidToTileset;
     QMap<const Tileset*, int> mTilesetColumnCounts;
 };
 
