@@ -139,7 +139,7 @@ void IsometricRenderer::drawGrid(QPainter *painter, const QRectF &rect,
 
     gridColor.setAlpha(128);
 
-    QPen gridPen(gridColor);
+    QPen gridPen(gridColor, 0);
     gridPen.setDashPattern(QVector<qreal>() << 2 << 2);
     painter->setPen(gridPen);
 
@@ -302,7 +302,7 @@ void IsometricRenderer::drawMapObject(QPainter *painter,
 {
     painter->save();
 
-    QPen pen(Qt::black);
+    QPen pen(Qt::black, 0);
 
     if (object->tile()) {
         const QPixmap &img = object->tile()->image();
@@ -495,14 +495,13 @@ QPointF IsometricRenderer::pixelToTileCoords(qreal x, qreal y) const
 {
     const int tileWidth = map()->tileWidth();
     const int tileHeight = map()->tileHeight();
-    const qreal ratio = (qreal) tileWidth / tileHeight;
 
     x -= map()->height() * tileWidth / 2;
-    const qreal mx = y + (x / ratio);
-    const qreal my = y - (x / ratio);
+    const qreal tileY = y / tileHeight;
+    const qreal tileX = x / tileWidth;
 
-    return QPointF(mx / tileHeight,
-                   my / tileHeight);
+    return QPointF(tileY + tileX,
+                   tileY - tileX);
 }
 
 QPointF IsometricRenderer::tileToPixelCoords(qreal x, qreal y) const
