@@ -178,6 +178,11 @@ public:
      */
     void merge(const QPoint &pos, const TileLayer *layer);
 
+    /**
+     * Removes all cells in the specified region.
+     */
+    void erase(const QRegion &region);
+
     /**
      * Sets the cells starting at the given position to the cells in the given
      * \a tileLayer. Parts that fall outside of this layer will be ignored.
@@ -263,8 +268,6 @@ public:
 
     virtual Layer *clone() const;
 
-    virtual TileLayer *asTileLayer() { return this; }
-
 protected:
     TileLayer *initializeClone(TileLayer *clone) const;
 
