@@ -34,14 +34,17 @@
 
 #include <QColor>
 #include <QList>
+#include <QVector>
 #include <QPoint>
 #include <QString>
+#include <QPixmap>
 
 class QImage;
 
 namespace Tiled {
 
 class Tile;
+class Terrain;
 
 /**
  * A tileset, representing a set of tiles.
@@ -70,7 +73,8 @@ public:
         mMargin(margin),
         mImageWidth(0),
         mImageHeight(0),
-        mColumnCount(0)
+        mColumnCount(0),
+        mTerrainDistancesDirty(false)
     {
         Q_ASSERT(tileSpacing >= 0);
         Q_ASSERT(margin >= 0);
@@ -108,15 +112,20 @@ public:
     bool isExternal() const { return !mFileName.isEmpty(); }
 
     /**
-     * Returns the width of the tiles in this tileset.
+     * Returns the maximum width of the tiles in this tileset.
      */
     int tileWidth() const { return mTileWidth; }
 
     /**
-     * Returns the height of the tiles in this tileset.
+     * Returns the maximum height of the tiles in this tileset.
      */
     int tileHeight() const { return mTileHeight; }
 
+    /**
+     * Returns the maximum size of the tiles in this tileset.
+     */
+    QSize tileSize() const { return QSize(mTileWidth, mTileHeight); }
+
     /**
      * Returns the spacing between the tiles in the tileset image.
      */
@@ -138,6 +147,11 @@ public:
      */
     void setTileOffset(QPoint offset) { mTileOffset = offset; }
 
+    /**
+     * Returns a const reference to the list of tiles in this tileset.
+     */
+    const QList<Tile*> &tiles() const { return mTiles; }
+
     /**
      * Returns the tile for the given tile ID.
      * The tile ID is local to this tileset, which means the IDs are in range
@@ -213,7 +227,84 @@ public:
      */
     int columnCountForWidth(int width) const;
 
+    /**
+     * Returns a const reference to the list of terrains in this tileset.
+     */
+    const QList<Terrain*> &terrains() const { return mTerrainTypes; }
+
+    /**
+     * Returns the number of terrain types in this tileset.
+     */
+    int terrainCount() const { return mTerrainTypes.size(); }
+
+    /**
+     * Returns the terrain type at the given \a index.
+     */
+    Terrain *terrain(int index) const { return index >= 0 ? mTerrainTypes[index] : 0; }
+
+    /**
+     * Adds a new terrain type.
+     *
+     * @param name      the name of the terrain
+     * @param imageTile the id of the tile that represents the terrain visually
+     * @return the created Terrain instance
+     */
+    Terrain *addTerrain(const QString &name, int imageTileId);
+
+    /**
+     * Adds the \a terrain type at the given \a index.
+     *
+     * The terrain should already have this tileset associated with it.
+     */
+    void insertTerrain(int index, Terrain *terrain);
+
+    /**
+     * Removes the terrain type at the given \a index and returns it. The
+     * caller becomes responsible for the lifetime of the terrain type.
+     *
+     * This will cause the terrain ids of subsequent terrains to shift up to
+     * fill the space and the terrain information of all tiles in this tileset
+     * will be updated accordingly.
+     */
+    Terrain *takeTerrainAt(int index);
+
+    /**
+     * Returns the transition penalty(/distance) between 2 terrains. -1 if no transition is possible.
+     */
+    int terrainTransitionPenalty(int terrainType0, int terrainType1);
+
+    /**
+     * Add a new tile to the end of the tileset
+     */
+    void addTile(const QPixmap &image);
+
+    /**
+     * Set a tile's image
+     */
+    void setTileImage(int index, const QPixmap &image);
+
+    /**
+     * Used by the Tile class when its terrain information changes.
+     */
+    void markTerrainDistancesDirty() { mTerrainDistancesDirty = true; }
+
 private:
+    /**
+     * Detaches from the external image. Should be called everytime the tileset
+     * is changed.
+     */
+    void detachExternalImage();
+
+    /**
+     * Sets tile size to the maximum size.
+     */
+    void updateTileSize();
+
+    /**
+     * Calculates the transition distance matrix for all terrain types.
+     */
+    void recalculateTerrainDistances();
+
     QString mName;
     QString mFileName;
     QString mImageSource;
@@ -227,6 +318,8 @@ private:
     int mImageHeight;
     int mColumnCount;
     QList<Tile*> mTiles;
+    QList<Terrain*> mTerrainTypes;
+    bool mTerrainDistancesDirty;
 };
 
 } // namespace Tiled
