@@ -29,6 +29,7 @@
 #include "tile.h"
 #include "tilelayer.h"
 #include "tileset.h"
+#include "terrain.h"
 
 using namespace Tiled;
 using namespace Json;
@@ -48,6 +49,10 @@ QVariant MapToVariantConverter::toVariant(const Map *map, const QDir &mapDir)
     mapVariant["tileheight"] = map->tileHeight();
     mapVariant["properties"] = toVariant(map->properties());
 
+    const QColor bgColor = map->backgroundColor();
+    if (bgColor.isValid())
+        mapVariant["backgroundcolor"] = bgColor.name();
+
     QVariantList tilesetVariants;
 
     unsigned firstGid = 1;
@@ -112,16 +117,43 @@ QVariant MapToVariantConverter::toVariant(const Tileset *tileset, int firstGid)
         tilesetVariant["imageheight"] = tileset->imageHeight();
     }
 
-    // Write the properties for those tiles that have them
+    // Write the properties & terrain for those tiles that have them
     QVariantMap tilePropertiesVariant;
+    QVariantMap tilesVariant;
     for (int i = 0; i < tileset->tileCount(); ++i) {
         const Tile *tile = tileset->tileAt(i);
         const Properties properties = tile->properties();
         if (!properties.isEmpty())
             tilePropertiesVariant[QString::number(i)] = toVariant(properties);
+        QVariantMap tileVariant;
+        if (tile->terrain() != 0xFFFFFFFF) {
+            QVariantList terrainIds;
+            for (int j = 0; j < 4; ++j)
+                terrainIds << QVariant(tile->cornerTerrainId(j));
+            tileVariant["terrain"] = terrainIds;
+        }
+        if (tile->terrainProbability() != -1.f)
+            tileVariant["probability"] = tile->terrainProbability();
+        if (!tileVariant.empty())
+            tilesVariant[QString::number(i)] = tileVariant;
     }
     if (!tilePropertiesVariant.empty())
         tilesetVariant["tileproperties"] = tilePropertiesVariant;
+    if (!tilesVariant.empty())
+        tilesetVariant["tiles"] = tilesVariant;
+
+    // Write terrains
+    if (tileset->terrainCount() > 0) {
+        QVariantList terrainsVariant;
+        for (int i = 0; i < tileset->terrainCount(); ++i) {
+            Terrain *terrain = tileset->terrain(i);
+            QVariantMap terrainVariant;
+            terrainVariant["name"] = terrain->name();
+            terrainVariant["tile"] = terrain->imageTileId();
+            terrainsVariant << terrainVariant;
+        }
+        tilesetVariant["terrains"] = terrainsVariant;
+    }
 
     return tilesetVariant;
 }
