@@ -58,6 +58,13 @@ Map *VariantToMapConverter::toMap(const QVariant &variant,
 
     mMap->setProperties(toProperties(variantMap["properties"]));
 
+    const QString bgColor = variantMap["backgroundcolor"].toString();
+    if (!bgColor.isEmpty())
+#if QT_VERSION >= 0x040700
+        if (QColor::isValidColor(bgColor))
+#endif
+            mMap->setBackgroundColor(QColor(bgColor));
+
     foreach (const QVariant &tilesetVariant, variantMap["tilesets"].toList()) {
         Tileset *tileset = toTileset(tilesetVariant);
         if (!tileset) {
@@ -125,7 +132,7 @@ Tileset *VariantToMapConverter::toTileset(const QVariant &variant)
     QString imageSource = variantMap["image"].toString();
 
     if (QDir::isRelativePath(imageSource))
-        imageSource = mMapDir.path() + QLatin1Char('/') + imageSource;
+        imageSource = QDir::cleanPath(mMapDir.absoluteFilePath(imageSource));
 
     if (!tileset->loadFromImage(QImage(imageSource), imageSource)) {
         mError = tr("Error loading tileset image:\n'%1'").arg(imageSource);
@@ -146,6 +153,36 @@ Tileset *VariantToMapConverter::toTileset(const QVariant &variant)
         }
     }
 
+    // Read terrains
+    QVariantList terrainsVariantList = variantMap["terrains"].toList();
+    for (int i = 0; i < terrainsVariantList.count(); ++i) {
+        QVariantMap terrainMap = terrainsVariantList[i].toMap();
+        tileset->addTerrain(terrainMap["name"].toString(),
+                            terrainMap["tile"].toInt());
+    }
+
+    // Read tile terrain information
+    const QVariantMap tilesVariantMap = variantMap["tiles"].toMap();
+    for (it = tilesVariantMap.begin(); it != tilesVariantMap.end(); ++it) {
+        bool ok;
+        const int tileIndex = it.key().toInt();
+        Tile *tile = tileset->tileAt(tileIndex);
+        if (tileIndex >= 0 && tileIndex < tileset->tileCount()) {
+            const QVariantMap tileVar = it.value().toMap();
+            QList<QVariant> terrains = tileVar["terrain"].toList();
+            if (terrains.count() == 4) {
+                for (int i = 0; i < 4; ++i) {
+                    int terrainID = terrains.at(i).toInt(&ok);
+                    if (ok && terrainID >= 0 && terrainID < tileset->terrainCount())
+                        tile->setCornerTerrain(i, terrainID);
+                }
+            }
+            float terrainProbability = tileVar["probability"].toFloat(&ok);
+            if (ok)
+                tile->setTerrainProbability(terrainProbability);
+        }
+    }
+
     mGidMapper.insert(firstGid, tileset);
     return tileset;
 }
@@ -334,8 +371,12 @@ ImageLayer *VariantToMapConverter::toImageLayer(const QVariantMap &variantMap)
 #endif
             imageLayer->setTransparentColor(QColor(trans));
 
-    const QString imageSource = variantMap["image"].toString();
+    QString imageSource = variantMap["image"].toString();
+
     if (!imageSource.isEmpty()) {
+        if (QDir::isRelativePath(imageSource))
+            imageSource = QDir::cleanPath(mMapDir.absoluteFilePath(imageSource));
+
         if (!imageLayer->loadFromImage(QImage(imageSource), imageSource)) {
             // TODO: This error is currently ignored
             mError = tr("Error loading image:\n'%1'").arg(imageSource);
