@@ -92,4 +92,6 @@ QString TmwPlugin::errorString() const
     return mError;
 }
 
+#if QT_VERSION < 0x050000
 Q_EXPORT_PLUGIN2(Tmw, TmwPlugin)
+#endif
