@@ -142,9 +142,5 @@ TileLayer *AbstractTileTool::currentTileLayer() const
     if (!mapDocument())
         return 0;
 
-    const int currentLayerIndex = mapDocument()->currentLayer();
-    if (currentLayerIndex < 0)
-        return 0;
-    Layer *currentLayer = mapDocument()->map()->layerAt(currentLayerIndex);
-    return dynamic_cast<TileLayer*>(currentLayer);
+    return dynamic_cast<TileLayer*>(mapDocument()->currentLayer());
 }
