@@ -131,6 +131,13 @@ void DocumentManager::switchToDocument(int index)
     mTabWidget->setCurrentIndex(index);
 }
 
+void DocumentManager::switchToDocument(MapDocument *mapDocument)
+{
+    const int index = mDocuments.indexOf(mapDocument);
+    if (index != -1)
+        switchToDocument(index);
+}
+
 void DocumentManager::switchToLeftDocument()
 {
     const int tabCount = mTabWidget->count();
@@ -182,8 +189,13 @@ void DocumentManager::closeCurrentDocument()
     if (index == -1)
         return;
 
+    closeDocumentAt(index);
+}
+
+void DocumentManager::closeDocumentAt(int index)
+{
     MapDocument *mapDocument = mDocuments.takeAt(index);
-    MapView *mapView = currentMapView();
+    QWidget *mapView = mTabWidget->widget(index);
 
     mTabWidget->removeTab(index);
     delete mapView;
