@@ -103,6 +103,7 @@ public:
      * Switches to the map document at the given \a index.
      */
     void switchToDocument(int index);
+    void switchToDocument(MapDocument *mapDocument);
 
     /**
      * Adds the new or opened \a mapDocument to the document manager.
@@ -115,6 +116,12 @@ public:
      */
     void closeCurrentDocument();
 
+    /**
+     * Closes the document at the given \a index. Will not ask the user whether
+     * to save any changes!
+     */
+    void closeDocumentAt(int index);
+
     /**
      * Close all documents. Will not ask the user whether to save any changes!
      */
