@@ -81,7 +81,7 @@
 #include "objectsdock.h"
 #include "minimapdock.h"
 
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
 #include "macsupport.h"
 #endif
 
@@ -127,7 +127,7 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     mUi->setupUi(this);
     setCentralWidget(mDocumentManager->widget());
 
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
     MacSupport::addFullscreen(this);
 #endif
 
@@ -136,8 +136,8 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     QIcon redoIcon(QLatin1String(":images/16x16/edit-redo.png"));
     QIcon undoIcon(QLatin1String(":images/16x16/edit-undo.png"));
 
-    QIcon tiledIcon(QLatin1String(":images/tiled-icon-16.png"));
-    tiledIcon.addFile(QLatin1String(":images/tiled-icon-32.png"));
+    QIcon tiledIcon(QLatin1String(":images/16x16/tiled.png"));
+    tiledIcon.addFile(QLatin1String(":images/32x32/tiled.png"));
     setWindowIcon(tiledIcon);
 
     // Add larger icon versions for actions used in the tool bar
@@ -547,9 +547,9 @@ bool MainWindow::openFile(const QString &fileName,
 
     TmxMapReader tmxMapReader;
 
+    const PluginManager *pm = PluginManager::instance();
     if (!mapReader && !tmxMapReader.supportsFile(fileName)) {
         // Try to find a plugin that implements support for this format
-        const PluginManager *pm = PluginManager::instance();
         QList<MapReaderInterface*> readers =
                 pm->interfaces<MapReaderInterface>();
 
@@ -563,10 +563,9 @@ bool MainWindow::openFile(const QString &fileName,
 
     // check if we can save in that format as well
     QString writerPluginFileName;
-    PluginManager *pm = PluginManager::instance();
     if (mapReader) {
-        if (dynamic_cast<MapWriterInterface*>(mapReader)) {
-            if (const Plugin *plugin = pm->plugin(mapReader))
+        if (const Plugin *plugin = pm->plugin(mapReader)) {
+            if (qobject_cast<MapWriterInterface*>(plugin->instance))
                 writerPluginFileName = plugin->fileName;
         }
     } else {
@@ -780,11 +779,13 @@ bool MainWindow::saveFileAs()
     return saveFile(fileName);
 }
 
-bool MainWindow::confirmSave()
+bool MainWindow::confirmSave(MapDocument *mapDocument)
 {
-    if (!mMapDocument || !mMapDocument->isModified())
+    if (!mapDocument || !mapDocument->isModified())
         return true;
 
+    mDocumentManager->switchToDocument(mapDocument);
+
     int ret = QMessageBox::warning(
             this, tr("Unsaved Changes"),
             tr("There are unsaved changes. Do you want to save now?"),
@@ -801,9 +802,8 @@ bool MainWindow::confirmSave()
 
 bool MainWindow::confirmAllSave()
 {
-    for (int i = 0; i < mDocumentManager->documentCount(); i++) {
-        mDocumentManager->switchToDocument(i);
-        if (!confirmSave())
+    for (int i = 0; i < mDocumentManager->documentCount(); ++i) {
+        if (!confirmSave(mDocumentManager->documents().at(i)))
             return false;
     }
 
@@ -916,7 +916,7 @@ void MainWindow::exportAs()
 
 void MainWindow::closeFile()
 {
-    if (confirmSave())
+    if (confirmSave(mDocumentManager->currentDocument()))
         mDocumentManager->closeCurrentDocument();
 }
 
@@ -1456,10 +1456,10 @@ void MainWindow::writeSettings()
     QStringList scrollY;
     QStringList selectedLayer;
     for (int i = 0; i < mDocumentManager->documentCount(); i++) {
-        mDocumentManager->switchToDocument(i);
-        fileList.append(mDocumentManager->currentDocument()->fileName());
-        MapView *mapView = mDocumentManager->currentMapView();
-        const int currentLayerIndex = mMapDocument->currentLayerIndex();
+        MapDocument *document = mDocumentManager->documents().at(i);
+        MapView *mapView = mDocumentManager->viewForDocument(document);
+        fileList.append(document->fileName());
+        const int currentLayerIndex = document->currentLayerIndex();
 
         mapScales.append(QString::number(mapView->zoomable()->scale()));
         scrollX.append(QString::number(
@@ -1600,7 +1600,6 @@ void MainWindow::setupQuickStamps()
 
 void MainWindow::closeMapDocument(int index)
 {
-    mDocumentManager->switchToDocument(index);
-    if (confirmSave())
-        mDocumentManager->closeCurrentDocument();
+    if (confirmSave(mDocumentManager->documents().at(index)))
+        mDocumentManager->closeDocumentAt(index);
 }
