@@ -55,10 +55,10 @@ public:
     QAction *actionCropToSelection() const { return mActionCropToSelection; }
     QAction *actionAddTileLayer() const { return mActionAddTileLayer; }
     QAction *actionAddObjectGroup() const { return mActionAddObjectGroup; }
+    QAction *actionAddImageLayer() const { return mActionAddImageLayer; }
     QAction *actionDuplicateLayer() const { return mActionDuplicateLayer; }
     QAction *actionMergeLayerDown() const { return mActionMergeLayerDown; }
     QAction *actionRemoveLayer() const { return mActionRemoveLayer; }
-    QAction *actionRenameLayer() const { return mActionRenameLayer; }
     QAction *actionSelectPreviousLayer() const
     { return mActionSelectPreviousLayer; }
     QAction *actionSelectNextLayer() const { return mActionSelectNextLayer; }
@@ -81,6 +81,7 @@ public slots:
 
     void addTileLayer();
     void addObjectGroup();
+    void addImageLayer();
     void duplicateLayer();
     void mergeLayerDown();
     void selectPreviousLayer();
@@ -88,7 +89,6 @@ public slots:
     void moveLayerUp();
     void moveLayerDown();
     void removeLayer();
-    void renameLayer();
     void toggleOtherLayers();
 
 private slots:
@@ -102,10 +102,10 @@ private:
     QAction *mActionCropToSelection;
     QAction *mActionAddTileLayer;
     QAction *mActionAddObjectGroup;
+    QAction *mActionAddImageLayer;
     QAction *mActionDuplicateLayer;
     QAction *mActionMergeLayerDown;
     QAction *mActionRemoveLayer;
-    QAction *mActionRenameLayer;
     QAction *mActionSelectPreviousLayer;
     QAction *mActionSelectNextLayer;
     QAction *mActionMoveLayerUp;
