@@ -22,6 +22,7 @@
 #define MAPVIEW_H
 
 #include <QGraphicsView>
+#include <QPinchGesture>
 
 namespace Tiled {
 namespace Internal {
@@ -62,6 +63,10 @@ protected:
     void mouseReleaseEvent(QMouseEvent *event);
     void mouseMoveEvent(QMouseEvent *event);
 
+    void handlePinchGesture(QPinchGesture *pinch);
+
+    void adjustCenterFromMousePosition(QPoint &mousePos);
+
 private slots:
     void adjustScale(qreal scale);
     void setUseOpenGL(bool useOpenGL);
