@@ -23,10 +23,13 @@
 #define OBJECTPROPERTIESDIALOG_H
 
 #include "propertiesdialog.h"
-#include "ui_objectpropertiesdialog.h"
 
 class QLineEdit;
 
+namespace Ui {
+class ObjectPropertiesDialog;
+}
+
 namespace Tiled {
 
 class MapObject;
