@@ -29,7 +29,6 @@ namespace Tiled {
 namespace Internal {
 
 class MapObjectItem;
-class MapScene;
 class SelectionRectangle;
 
 class ObjectSelectionTool : public AbstractObjectTool
@@ -49,9 +48,6 @@ public:
 
     void languageChanged();
 
-protected:
-    void updateEnabledState();
-
 private:
     enum Mode {
         NoMode,
