@@ -21,8 +21,6 @@
 #ifndef PREFERENCESDIALOG_H
 #define PREFERENCESDIALOG_H
 
-#include "map.h"
-
 #include <QDialog>
 
 class QModelIndex;
@@ -68,8 +66,6 @@ private:
     void fromPreferences();
     void toPreferences();
 
-    Map::LayerDataFormat layerDataFormat() const;
-
     Ui::PreferencesDialog *mUi;
     QStringList mLanguages;
     ObjectTypesModel *mObjectTypesModel;
