@@ -33,12 +33,8 @@ bool TmxMapWriter::write(const Map *map, const QString &fileName)
 {
     Preferences *prefs = Preferences::instance();
 
-    Map::LayerDataFormat format = map->layerDataFormat();
-    if (format == Map::Default)
-        format = prefs->layerDataFormat();
-
     MapWriter writer;
-    writer.setLayerDataFormat(format);
+    writer.setLayerDataFormat(map->layerDataFormat());
     writer.setDtdEnabled(prefs->dtdEnabled());
 
     bool result = writer.writeMap(map, fileName);
