@@ -28,7 +28,9 @@
 enum UndoCommands {
     Cmd_EraseTiles,
     Cmd_PaintTileLayer,
-    Cmd_MoveTileset
+    Cmd_MoveTileset,
+    Cmd_ChangeLayerOpacity,
+    Cmd_ChangeTileTerrain
 };
 
 #endif // UNDOCOMMANDS_H
