@@ -106,18 +106,6 @@ public class Tile
         return id;
     }
 
-    /**
-     * Returns the global tile id by adding the tile id to the map-assigned.
-     *
-     * @return id
-     */
-    public int getGid() {
-        if (tileset != null) {
-            return id + tileset.getFirstGid();
-        }
-        return id;
-    }
-
     /**
      * Returns the {@link tiled.core.TileSet} that this tile is part of.
      *
