@@ -36,7 +36,7 @@ import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.net.MalformedURLException;
 import java.net.URL;
-import java.util.Properties;
+import java.util.*;
 import java.util.zip.GZIPInputStream;
 import java.util.zip.InflaterInputStream;
 import javax.imageio.ImageIO;
@@ -51,6 +51,7 @@ import org.xml.sax.EntityResolver;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
 import tiled.core.*;
+import tiled.core.Map;
 import tiled.util.Base64;
 import tiled.util.BasicTileCutter;
 import tiled.util.ImageHelper;
@@ -64,6 +65,13 @@ public class TMXMapReader
     private String xmlPath;
     private String error;
     private final EntityResolver entityResolver = new MapEntityResolver();
+    private TreeMap<Integer, TileSet> tilesetPerFirstGid;
+    public final TMXMapReaderSettings settings = new TMXMapReaderSettings();
+    private final HashMap<String, TileSet> cachedTilesets = new HashMap<String, TileSet>();
+
+    public static final class TMXMapReaderSettings {
+        public boolean reuseCachedTilesets = false;
+    }
 
     public TMXMapReader() {
     }
@@ -299,7 +307,7 @@ public class TMXMapReader
             try {
                 InputStream in = new URL(makeUrl(filename)).openStream();
                 ext = unmarshalTilesetFile(in, filename);
-                ext.setFirstGid(firstGid);
+                setFirstGidForTileset(ext, firstGid);
             } catch (FileNotFoundException fnf) {
                 error = "Could not find external tileset file " + filename;
             }
@@ -316,11 +324,24 @@ public class TMXMapReader
             final int tileSpacing = getAttribute(t, "spacing", 0);
             final int tileMargin = getAttribute(t, "margin", 0);
 
-            TileSet set = new TileSet();
+            final String name = getAttributeValue(t, "name");
 
-            set.setName(getAttributeValue(t, "name"));
+            TileSet set;
+            if (settings.reuseCachedTilesets) {
+                set = cachedTilesets.get(name);
+                if (set != null) {
+                    setFirstGidForTileset(set, firstGid);
+                    return set;
+                }
+                set = new TileSet();
+                cachedTilesets.put(name, set);
+            } else {
+                set = new TileSet();
+            }
+
+            set.setName(name);
             set.setBaseDir(basedir);
-            set.setFirstGid(firstGid);
+            setFirstGidForTileset(set, firstGid);
 
             boolean hasTilesetImage = false;
             NodeList children = t.getChildNodes();
@@ -567,9 +588,12 @@ public class TMXMapReader
                         String comp = getAttributeValue(child, "compression");
 
                         if ("gzip".equalsIgnoreCase(comp)) {
-                            is = new GZIPInputStream(bais);
+                            final int len = layerWidth * layerHeight * 4;
+                            is = new GZIPInputStream(bais, len);
                         } else if ("zlib".equalsIgnoreCase(comp)) {
                             is = new InflaterInputStream(bais);
+                        } else if (comp != null && !comp.isEmpty()) {
+                            throw new IOException("Unrecognized compression method \"" + comp + "\" for map layer " + ml.getName());
                         } else {
                             is = bais;
                         }
@@ -582,10 +606,10 @@ public class TMXMapReader
                                 tileId |= is.read() << 16;
                                 tileId |= is.read() << 24;
 
-                                TileSet ts = map.findTileSetForTileGID(tileId);
+                                java.util.Map.Entry<Integer, TileSet> ts = findTileSetForTileGID(tileId);
                                 if (ts != null) {
                                     ml.setTileAt(x, y,
-                                            ts.getTile(tileId - ts.getFirstGid()));
+                                            ts.getValue().getTile(tileId - ts.getKey()));
                                 } else {
                                     ml.setTileAt(x, y, null);
                                 }
@@ -600,10 +624,10 @@ public class TMXMapReader
                     {
                         if ("tile".equalsIgnoreCase(dataChild.getNodeName())) {
                             int tileId = getAttribute(dataChild, "gid", -1);
-                            TileSet ts = map.findTileSetForTileGID(tileId);
+                            java.util.Map.Entry<Integer, TileSet> ts = findTileSetForTileGID(tileId);
                             if (ts != null) {
                                 ml.setTileAt(x, y,
-                                        ts.getTile(tileId - ts.getFirstGid()));
+                                        ts.getValue().getTile(tileId - ts.getKey()));
                             } else {
                                 ml.setTileAt(x, y, null);
                             }
@@ -704,6 +728,7 @@ public class TMXMapReader
         readProperties(mapNode.getChildNodes(), map.getProperties());
 
         // Load tilesets first, in case order is munged
+        tilesetPerFirstGid = new TreeMap<Integer, TileSet>();
         NodeList l = doc.getElementsByTagName("tileset");
             for (int i = 0; (item = l.item(i)) != null; i++) {
                     map.addTileset(unmarshalTileset(item));
@@ -726,6 +751,7 @@ public class TMXMapReader
                 }
             }
         }
+        tilesetPerFirstGid = null;
     }
 
     private Map unmarshal(InputStream in) throws Exception {
@@ -748,6 +774,7 @@ public class TMXMapReader
         }
 
         buildMap(doc);
+
         return map;
     }
 
@@ -849,4 +876,21 @@ public class TMXMapReader
 
         return false;
     }
+
+    /**
+     * Get the tile set and its corresponding firstgid that matches the given
+     * global tile id.
+     *
+     *
+     * @param gid a global tile id
+     * @return the tileset containing the tile with the given global tile id,
+     *         or <code>null</code> when no such tileset exists
+     */
+    private java.util.Map.Entry<Integer, TileSet> findTileSetForTileGID(int gid) {
+        return tilesetPerFirstGid.floorEntry(gid);
+    }
+
+    private void setFirstGidForTileset(TileSet tileset, int firstGid) {
+        tilesetPerFirstGid.put(firstGid, tileset);
+    }
 }
