@@ -33,6 +33,7 @@ import java.awt.Rectangle;
 import java.io.*;
 import java.nio.charset.Charset;
 import java.util.*;
+import java.util.zip.DeflaterOutputStream;
 import java.util.zip.GZIPOutputStream;
 
 import tiled.core.*;
@@ -50,6 +51,16 @@ public class TMXMapWriter
     private static final boolean encodeLayerData = true;
     private static final boolean compressLayerData = encodeLayerData;
 
+    private HashMap<String, Integer> firstGidPerTileset;
+
+    public static class Settings {
+        public static final String LAYER_COMPRESSION_METHOD_GZIP = "gzip";
+        public static final String LAYER_COMPRESSION_METHOD_ZLIB = "zlib";
+
+        public String layerCompressionMethod = LAYER_COMPRESSION_METHOD_GZIP;
+    }
+    public Settings settings = new Settings();
+
     /**
      * Saves a map to an XML file.
      *
@@ -116,7 +127,7 @@ public class TMXMapWriter
         writer.flush();
     }
 
-    private static void writeMap(Map map, XMLWriter w, String wp) throws IOException {
+    private void writeMap(Map map, XMLWriter w, String wp) throws IOException {
         w.writeDocType("map", null, "http://mapeditor.org/dtd/1.0/map.dtd");
         w.startElement("map");
 
@@ -140,9 +151,10 @@ public class TMXMapWriter
 
         writeProperties(map.getProperties(), w);
 
+        firstGidPerTileset = new HashMap<String, Integer>();
         int firstgid = 1;
         for (TileSet tileset : map.getTileSets()) {
-            tileset.setFirstGid(firstgid);
+            setFirstGidForTileset(tileset, firstgid);
             writeTilesetReference(tileset, w, wp);
             firstgid += tileset.getMaxTileId() + 1;
         }
@@ -150,6 +162,7 @@ public class TMXMapWriter
         for (MapLayer layer : map) {
             writeMapLayer(layer, w, wp);
         }
+        firstGidPerTileset = null;
 
         w.endElement();
     }
@@ -188,7 +201,7 @@ public class TMXMapWriter
      * @param wp  the working directory of the map
      * @throws java.io.IOException
      */
-    private static void writeTilesetReference(TileSet set, XMLWriter w, String wp)
+    private void writeTilesetReference(TileSet set, XMLWriter w, String wp)
         throws IOException {
 
         String source = set.getSource();
@@ -197,7 +210,7 @@ public class TMXMapWriter
             writeTileset(set, w, wp);
         } else {
             w.startElement("tileset");
-            w.writeAttribute("firstgid", set.getFirstGid());
+            w.writeAttribute("firstgid", getFirstGidForTileset(set));
             w.writeAttribute("source", getRelativePath(wp, source));
             if (set.getBaseDir() != null) {
                 w.writeAttribute("basedir", set.getBaseDir());
@@ -206,14 +219,14 @@ public class TMXMapWriter
         }
     }
 
-    private static void writeTileset(TileSet set, XMLWriter w, String wp)
+    private void writeTileset(TileSet set, XMLWriter w, String wp)
         throws IOException {
 
         String tileBitmapFile = set.getTilebmpFile();
         String name = set.getName();
 
         w.startElement("tileset");
-        w.writeAttribute("firstgid", set.getFirstGid());
+        w.writeAttribute("firstgid", getFirstGidForTileset(set));
 
         if (name != null) {
             w.writeAttribute("name", name);
@@ -297,7 +310,7 @@ public class TMXMapWriter
      * first global ids for the tilesets are determined, in order for the right
      * gids to be written to the layer data.
      */
-    private static void writeMapLayer(MapLayer l, XMLWriter w, String wp) throws IOException {
+    private void writeMapLayer(MapLayer l, XMLWriter w, String wp) throws IOException {
         Rectangle bounds = l.getBounds();
 
         if (l instanceof ObjectGroup) {
@@ -307,8 +320,12 @@ public class TMXMapWriter
         }
 
         w.writeAttribute("name", l.getName());
-        w.writeAttribute("width", bounds.width);
-        w.writeAttribute("height", bounds.height);
+        if (bounds.width != 0) {
+            w.writeAttribute("width", bounds.width);
+        }
+        if (bounds.height != 0) {
+            w.writeAttribute("height", bounds.height);
+        }
         if (bounds.x != 0) {
             w.writeAttribute("x", bounds.x);
         }
@@ -336,9 +353,17 @@ public class TMXMapWriter
 
                 w.writeAttribute("encoding", "base64");
 
+                DeflaterOutputStream dos;
                 if (compressLayerData) {
-                    w.writeAttribute("compression", "gzip");
-                    out = new GZIPOutputStream(baos);
+                    if (Settings.LAYER_COMPRESSION_METHOD_ZLIB.equalsIgnoreCase(settings.layerCompressionMethod)) {
+                        dos = new DeflaterOutputStream(baos);
+                    } else if (Settings.LAYER_COMPRESSION_METHOD_GZIP.equalsIgnoreCase(settings.layerCompressionMethod)) {
+                        dos = new GZIPOutputStream(baos);
+                    } else {
+                        throw new IOException("Unrecognized compression method \"" + settings.layerCompressionMethod + "\" for map layer " + l.getName());
+                    }
+                    out = dos;
+                    w.writeAttribute("compression", settings.layerCompressionMethod);
                 } else {
                     out = baos;
                 }
@@ -350,7 +375,7 @@ public class TMXMapWriter
                         int gid = 0;
 
                         if (tile != null) {
-                            gid = tile.getGid();
+                            gid = getGid(tile);
                         }
 
                         out.write(gid       & LAST_BYTE);
@@ -360,8 +385,8 @@ public class TMXMapWriter
                     }
                 }
 
-                if (compressLayerData) {
-                    ((GZIPOutputStream)out).finish();
+                if (compressLayerData && dos != null) {
+                    dos.finish();
                 }
 
                 w.writeCDATA(Base64.encodeToString(baos.toByteArray(), true));
@@ -372,7 +397,7 @@ public class TMXMapWriter
                         int gid = 0;
 
                         if (tile != null) {
-                            gid = tile.getGid();
+                            gid = getGid(tile);
                         }
 
                         w.startElement("tile");
@@ -419,7 +444,7 @@ public class TMXMapWriter
      * @param w the writer to write to
      * @throws IOException when an io error occurs
      */
-    private static void writeTile(Tile tile, XMLWriter w) throws IOException {
+    private void writeTile(Tile tile, XMLWriter w) throws IOException {
         w.startElement("tile");
         w.writeAttribute("id", tile.getId());
 
@@ -431,7 +456,7 @@ public class TMXMapWriter
         w.endElement();
     }
 
-    private static void writeAnimation(Sprite s, XMLWriter w) throws IOException {
+    private void writeAnimation(Sprite s, XMLWriter w) throws IOException {
         w.startElement("animation");
         for (int k = 0; k < s.getTotalKeys(); k++) {
             Sprite.KeyFrame key = s.getKey(k);
@@ -440,7 +465,7 @@ public class TMXMapWriter
             for (int it = 0; it < key.getTotalFrames(); it++) {
                 Tile stile = key.getFrame(it);
                 w.startElement("tile");
-                w.writeAttribute("gid", stile.getGid());
+                w.writeAttribute("gid", getGid(stile));
                 w.endElement();
             }
             w.endElement();
@@ -560,4 +585,25 @@ public class TMXMapWriter
         } catch (IOException e) {}
         return false;
     }
+
+    /**
+     * Returns the global tile id of the given tile.
+     *
+     * @return global tile id of the given tile
+     */
+    private int getGid(Tile tile) {
+        TileSet tileset = tile.getTileSet();
+        if (tileset != null) {
+            return tile.getId() + getFirstGidForTileset(tileset);
+        }
+        return tile.getId();
+    }
+
+    private void setFirstGidForTileset(TileSet tileset, int firstGid) {
+        firstGidPerTileset.put(tileset.getName(), firstGid);
+    }
+
+    private int getFirstGidForTileset(TileSet tileset) {
+        return firstGidPerTileset.get(tileset.getName());
+    }
 }
